import pytest
from pydantic_forms.exceptions import FormValidationError

from gso.products.product_types.site import Site
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.mark.workflow()
def test_modify_site(responses, site_subscription_factory):
    subscription_id = site_subscription_factory()
    initial_site_data = [
        {"subscription_id": subscription_id},
        {
            "site_bgp_community_id": 10,
            "site_internal_id": 20,
            "site_ts_address": "127.0.0.1",
        },
    ]
    result, process, step_log = run_workflow("modify_site", initial_site_data)
    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Site.from_subscription(subscription_id)
    assert subscription.status == "active"
    assert subscription.site.site_bgp_community_id == initial_site_data[1]["site_bgp_community_id"]
    assert subscription.site.site_internal_id == initial_site_data[1]["site_internal_id"]


@pytest.mark.workflow()
def test_modify_site_with_invalid_data(responses, site_subscription_factory):
    subscription_a = Site.from_subscription(site_subscription_factory())
    subscription_b = Site.from_subscription(site_subscription_factory())

    initial_site_data = [
        {"subscription_id": subscription_b.subscription_id},
        {
            "site_bgp_community_id": subscription_a.site.site_bgp_community_id,
        },
    ]

    with pytest.raises(FormValidationError, match="site_bgp_community_id must be unique"):
        run_workflow("modify_site", initial_site_data)
